/***************
 * NAME:            "IconSplit" - version 1.3.0
 * DESCRIPTION:     Functions to split classic icon images
 * AUTHOR:          (c) Mike Riddle - initial release 03-06-23                     
 ***************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <zlib.h>

#include <dos/dos.h>
#include <exec/exec.h>

#include <proto/wb.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/icon.h>
#include <proto/intuition.h>

#include <workbench/icon.h>
#include <workbench/workbench.h>
#include <intuition/intuition.h>

#include <proto/datatypes.h>
#include <datatypes/datatypes.h>
#include <datatypes/pictureclass.h>



/*********************************/
/*    ADAPTED FROM MODIFYICON    */
/*********************************/

#define MAXTOOLTYPES 256
#define MAXICONSIZE 256
#define MAXDEPTH 8
#define MAXCOLORS (1<<MAXDEPTH)

//#define MAX_IMAGE_WIDTH 80
//#define MAX_IMAGE_HEIGHT 80

//#define ID_FORM MAKE_ID('F','O','R','M')
//#define ID_ICON MAKE_ID('I','C','O','N')
//#define ID_FACE MAKE_ID('F','A','C','E')
//#define ID_IMAG MAKE_ID('I','M','A','G')
//#define ID_ARGB MAKE_ID('A','R','G','B')

/* Defines for Get Data Functions - based on modifyicon */
#define GetUint32(a)  (((((unsigned char *)a)[0])<<24)|((((unsigned char *)a)[1])<<16)| \
                       ((((unsigned char *)a)[2])<< 8)|((((unsigned char *)a)[3])))
#define GetUint16(a)  (((((unsigned char *)a)[0])<< 8)|((((unsigned char *)a)[1])))

/* Defines for Put Data Functions - based on modifyicon */
#define PutUint32(a, b) {unsigned long epu32 = (b); (a)[0] = (unsigned char) (epu32 >> 24); \
                         (a)[1] = (unsigned char) (epu32 >> 16); \
                         (a)[2] = (unsigned char) (epu32 >> 8); \
                         (a)[3] = (unsigned char) epu32;}
#define PutUint16(a, b) {unsigned short epu16 = (b); (a)[0] = (unsigned char) (epu16 >> 8); \
                         (a)[1] = (unsigned char) epu16;}

#define PutUint32Inc(a, b) {PutUint32(a,b); (a) += 4;}
#define PutUint16Inc(a, b) {PutUint16(a,b); (a) += 2;}

//#define GetUint16BE(a)  (((((unsigned char *)a)[0]))|((((unsigned char *)a)[1])<< 8))
//#define GetUint16LE(a)  (((((unsigned char *)a)[0])>> 8)|((((unsigned char *)a)[1])))

enum DecodeType { DECODE_PALETTE, DECODE_IMAGE };

STRPTR srcename = NULL;

/* Open library allocations */
//struct Library *IconBase = NULL;

/****************************************************************************************/

/*********************************/
/*    ADAPTED FROM MODIFYICON    */
/*********************************/

struct IconPalette 
{
  unsigned char      Red;
  unsigned char      Green;
  unsigned char      Blue;
};

struct IconImage 
{
  LONG               Depth;
  LONG               NumColors;
  LONG               Width;
  LONG               Height;
  LONG               Transparency;     /* Transparent Color or -1 */
  /* Either 8bit or 32bit Chunky - depending on Colors = 0 or not */
  unsigned char      ImageData[MAXICONSIZE*MAXICONSIZE*MAXDEPTH/8];  
  struct IconPalette Palette[MAXCOLORS]; /* Image Color Palette */ 
};

struct IconImageCollection 
{
  char *IconStyle;
  struct IconImage NewImage[2];
  //struct IconImage *OS35Image[2];
  //struct IconImage *ARGBImage[2];
};

/****************************************************************************************/

LONG GetFileSize(char *imName);
LONG Get_IFFOffset(char *srcename); 
static char *ReadIcon(char *srcename);
static LONG ReadLong(BPTR file, LONG offset);
static WORD ReadWord(BPTR file, LONG offset);
BOOL ReadIconImage_IFF(char *srcename, LONG iffoffset);
BOOL ReadIconImage_PNG(char *srcename);
LONG Process_PNGIcon(char *name, int *index, LONG pOffset, LONG endFile);
static char *Process_Argb(char *filename, struct IconImage *im, LONG psize, LONG index);
static char *Process_NewIcon(char **tt_array);
static char *Process_NewIconImage(unsigned char **tt_array, struct IconImage *im, unsigned char num);
static void RemoveToolType(unsigned char **tt);
LONG Get_Tooltypes(char *srcename);
static BOOL LoadIconImage(char *name, struct IconImage *im);
static BOOL SaveImage(char *name, struct IconImage *im);
static char *SaveTarga(char *name, struct IconImage *im, int bpp);
static char *WriteTruecolor(BPTR file, struct IconImage *im, int bpp, BOOL tgacomp);
static BOOL SaveIconImage(char *name, struct IconImage *im, char *type);
static void WriteData(char *filename, unsigned char *data, LONG datasize);
static void SetPalette(struct IconImage *im, const struct IconPalette *pal, long num);
int ChunkyBytesToBitplanes(UBYTE *chunkyBuffer, UBYTE *bitBuffer, LONG bitPlanes, LONG imagewidth);
static void MakePlanar(unsigned char *inbuffer, unsigned char *outbuffer, LONG width, LONG height, LONG depth);
static char *DecodeNewIcon(unsigned char **tt, struct IconImage *im, enum DecodeType type, unsigned char num);
//static char *DecodeArgb(unsigned char *buffer, long numbytes, long psize, struct IconImage *im);
static char *Decode35(unsigned char *buffer, unsigned char *outbuffer, LONG numbytes, LONG bits, LONG entries);
static char *DecodeNew(unsigned char *buffer, long numbytes, long bits, struct IconImage *im, enum DecodeType type, long mode);
//static char *DecodeRLE(unsigned char *srce, unsigned long bytes, unsigned long len, struct IconImage *im);
static char *DecodeBytes(unsigned char *srce, unsigned long bytes, unsigned long depth, struct IconImage *im, unsigned long len);

/**************************************************************************************************/

static LONG ReadLong(BPTR file, LONG offset)
{
    WORD ret; //LONG ??
    UBYTE Buffer32[4];
    
    Seek(file,offset,OFFSET_BEGINNING);
	Read(file,Buffer32, 4);	    		
	ret = GetUint32(Buffer32); 
    
    return ret;
}

/**************************************************************************************************/

static WORD ReadWord(BPTR file, LONG offset)
{
    WORD ret;
    UBYTE Buffer16[2];
    
    Seek(file,offset,OFFSET_BEGINNING);
	Read(file,Buffer16, 2);	    		
	ret = GetUint16(Buffer16); 
    
    return ret;
}

/**************************************************************************************************/

LONG GetFileSize(char *imName)
{
    BPTR file = Open(imName, MODE_OLDFILE);
    struct FileInfoBlock *fib = (struct FileInfoBlock*)AllocDosObject( DOS_FIB, NULL );
    LONG size = -1;
    
    if (fib)
    {
	    if (ExamineFH( file, fib ))
	    {
	        if ((fib->fib_DirEntryType > 0) && (fib->fib_DirEntryType != ST_SOFTLINK))
	        {
		        size = 0;
	        }
	        size = fib->fib_Size;
	    }
	    FreeDosObject( DOS_FIB, fib );
    }
    Close(file);
    
    return size;
}

/**************************************************************************************************/

static char *ReadIcon(char *srcename)
{
    char *icontype;
    unsigned char buffer[4];

    if(srcename)
    {
        BPTR file = Open(srcename, MODE_OLDFILE);
        if(file)
        {
            Read(file,buffer, 4);
            if(GetUint32(buffer) == 0x89504E47)
                icontype = "png";
            else if(GetUint16(buffer) == 0xE310)
                icontype = "iff";
            else
                icontype = "none";
            Close(file);
        }
    }

    return icontype;
} 

/**************************************************************************************************/

static void WriteData(char *new_filename, unsigned char *data, LONG datasize)
{  
    BPTR fh = Open( new_filename, MODE_NEWFILE );
	if(fh)
	{        
	    Seek(fh,0,OFFSET_BEGINNING);
        Write(fh, data, datasize);
	}	
    Close( fh );
}

/**************************************************************************************************/

LONG Get_Tooltypes(char *srcename)
{	
	LONG offset;
    LONG current;
	LONG ttoffset;    
	LONG i, j, k;	
	UWORD FileID;		
	UBYTE Buffer16[2];
	UBYTE Buffer32[4];
    BOOL Found = FALSE;
    unsigned char *ttbuffer;	
	WORD width, height, depth;
	int imagesize, imageoffset;    

	//Printf("\nGet_Tooltypes...\n");
	
	LONG firstimage, secondimage;	
	LONG tooltypes, deftool, drawerdata, toolwin;
    char*ToolTypes[MAXTOOLTYPES];	

	/* Get total icon file size */
	LONG imSize = GetFileSize(srcename);
	//Printf ("File Size: %ld\n", imSize);

	/* Get byte array of IFF data */
    ttbuffer = (unsigned char *)AllocVec(imSize, MEMF_ANY);

	/* Open icon file to Read */
    BPTR fh = Open(srcename, MODE_OLDFILE);
    if (fh)
	{
		offset = 0;		     		  

        Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer16, 2);	    		
		if( Buffer16[0] == 0xE3 && Buffer16[1] == 0x10 ) 
		//Printf ("Found Classic Icon File!\n");

		offset += 2;
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer16, 2);		
		FileID = GetUint16(Buffer16);
		if(FileID == 1)
		//Printf("Found Valid Version Number!\n");

		offset += 20; //22
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);		
		firstimage = (GetUint32(Buffer32)==0) ? 0: 1; //22		

		offset += 4; //26
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);		
		secondimage = (GetUint32(Buffer32)==0) ? 0: 1; //26
		
		offset += 24; //26+24=50
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);		
		deftool = (GetUint32(Buffer32)==0) ? 0: 1; //50		
		offset += 4; //54
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);		
		tooltypes = (GetUint32(Buffer32)==0) ? 0: 1; //54        
		offset += 12; //66
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);        
		drawerdata = (GetUint32(Buffer32)==0) ? 0: 1; //66		       
		offset += 4; //70
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);		
		toolwin = (GetUint32(Buffer32)==0) ? 0: 1; //70		
		offset += 8; //64+14=78
		//Printf("Current Offset: %ld\n", offset); //78 ??

		if(drawerdata == 1)
        {		    
		    offset += 56; //78+56=134
		}
		if(firstimage == 1)
		{
			offset += 4;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
            width = GetUint16(Buffer16); //offset + 4			           
			//Printf ("First Image Width: %ld, 0x%x%x, %ld\n", width, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			height = GetUint16(Buffer16); //offset + 6			
			//Printf ("First Image Height: %ld, 0x%x%x, %ld\n", height, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			depth = GetUint16(Buffer16); //offset + 8			
			//Printf ("First Image Depth: %ld, 0x%x%x, %ld\n", depth, Buffer16[0],Buffer16[1], offset);

			offset += 12; //offset + 20
			imagesize = (((width+15)&(~15)) * height);
			//imageoffset = (((((width+15)&(~15)) * height) * depth)>>3);
			imageoffset = ((imagesize * depth)>>3);

			//Printf ("First Image Size: %ld, %ld\n", imageoffset, offset);
			offset += imageoffset;
		}
		if(secondimage == 1)
		{
			offset += 4;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
            width = GetUint16(Buffer16); //offset + 4			           
			//Printf ("Second Image Width: %ld, 0x%x%x, %ld\n", width, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			height = GetUint16(Buffer16); //offset + 6			
			//Printf ("Second Image Height: %ld, 0x%x%x, %ld\n", height, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			depth = GetUint16(Buffer16); //offset + 8			
			//Printf ("Second Image Depth: %ld, 0x%x%x, %ld\n", depth, Buffer16[0],Buffer16[1], offset);
			
			offset += 12; //offset + 20
			imagesize = (((width+15)&(~15)) * height);
			//imageoffset = (((((width+15)&(~15)) * height) * depth)>>3);
			imageoffset = ((imagesize * depth)>>3);						

			//Printf ("Second Image Size: %ld, %ld\n", imageoffset, offset);
			offset += imageoffset;
		}
		if(deftool == 1)
        {
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
			i = GetUint32(Buffer32);            
			offset += i+4;
			//Printf("Current Offset: %ld\n", offset);
		}
		if(tooltypes == 1)
		{
            current = offset;
			Printf("Tooltypes Begin Offset: %ld\n", offset);
            Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
			i = GetUint32(Buffer32)/4-1;
			//Printf("Tooltypes i value: %ld\n", i);
			offset += 4;
			
			for(j = 0; j < i; ++j)
            {
				Seek(fh,offset,OFFSET_BEGINNING);
		        Read(fh,Buffer32, 4);
                k = GetUint32(Buffer32);
                //Printf("Tooltypes k value: %ld\n", k);
				offset += k+4;
            }
			Printf("Tooltypes End Offset: %ld\n", offset);
			LONG ttsize = (offset - current);
            Printf("Size of Tooltypes in Bytes: %ld\n", ttsize);
			Seek(fh,current,OFFSET_BEGINNING);
            Read(fh,ttbuffer, ttsize);           
            
            //i = GetUint32(ttbuffer)/4-1;
            ttbuffer += 4; //skip the 'i' value           
            for(j = 0; j < i; ++j)
            {
                k = GetUint32(ttbuffer);                
                ToolTypes[j] = ttbuffer+4; //skip the 'k' value
                
                if(strcmp(ToolTypes[j], "*** DON'T EDIT THE FOLLOWING LINES!! ***") == 0) 
                {
                    Printf("Found New Icon Header in Tooltypes\n");                    
                    printf ("Tooltype: '%s'\n\n", ToolTypes[j]);                    
                    Found = TRUE;                        
                }                      
                ttbuffer += k+4;                
            }
                //Call Process_NewIcon
                if(Found == TRUE)
                //Printf ("Found = TRUE. Call Process_NewIcon\n");
                Process_NewIcon(ToolTypes);        
		}
        
        		
	}
	Close(fh);	
    //Printf("Current Offset: %ld\n", offset);

	return current;
    //return ttbuffer;
}

/**************************************************************************************************/

LONG Get_IFFOffset(char *srcename)
{
    //**********************************************//
    //   REVISED FUNCTION USING ReadLong/ReadWord   //
    //**********************************************//
    	
	LONG offset;
	LONG imSize;
	LONG i, j, k;	
	UWORD FileID;		
	UBYTE Buffer16[2];
	UBYTE Buffer32[4];	
	WORD width, height, depth;
	int imagesize, imageoffset;

	//Printf("\nGet_IFFOffset...\n");

	//unsigned char *buffer;
	LONG firstimage, secondimage;
    LONG drawerflags, drawerviewmode;	
	LONG tooltypes, deftool, drawerdata, toolwin;	

	/* Get total icon file size */
	imSize = GetFileSize(srcename);
	//Printf ("File Size: %ld\n", imSize);

	/* Get byte array of IFF data */
    //buffer = (unsigned char *)AllocVec(imSize, MEMF_ANY);

	/* Open icon file to Read */
    BPTR fh = Open(srcename, MODE_OLDFILE);
    if (fh)
	{
		offset = 0;
        
        Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer16, 2);	    		
		if( Buffer16[0] == 0xE3 && Buffer16[1] == 0x10 ) 
		Printf ("Found Classic Icon File!\n");

		offset += 2;
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer16, 2);		
		FileID = GetUint16(Buffer16);
		if(FileID == 1)
		Printf("Found Valid Version Number!\n");

        offset += 20; //22        
        firstimage = (ReadLong(fh,offset)==0) ? 0: 1; //22
        //Printf ("First Image: %ld, %ld\n", firstimage, offset); //GadgetRender
        offset += 4; //26
        secondimage = (ReadLong(fh,offset)==0) ? 0: 1; //26
        //Printf ("Second Image: %ld, %ld\n", secondimage, offset); //GadgetSelect
        offset += 24; //26+24=50
        deftool = (ReadLong(fh,offset)==0) ? 0: 1; //50
        offset += 4; //54
        tooltypes = (ReadLong(fh,offset)==0) ? 0: 1; //54
        offset += 12; //66
        drawerdata = (ReadLong(fh,offset)==0) ? 0: 1; //66
        offset += 4; //70
        toolwin = (ReadLong(fh,offset)==0) ? 0: 1; //70

        offset += 8; //64+14=78
		//Printf("DiskObject - Current Offset: %ld\n", offset); //78 ??
        

		if(drawerdata == 1)
        {		    
		    offset += 56; //78+56=134
            //Printf("DrawerData - Current Offset: %ld\n", offset);
		}
		if(firstimage == 1)
		{
			offset += 4;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
            width = GetUint16(Buffer16); //offset + 4			           
			//Printf ("First Image Width: %ld, 0x%x%x, %ld\n", width, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			height = GetUint16(Buffer16); //offset + 6			
			//Printf ("First Image Height: %ld, 0x%x%x, %ld\n", height, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			depth = GetUint16(Buffer16); //offset + 8
            //Printf ("First Image Depth: %ld, 0x%x%x, %ld\n", depth, Buffer16[0],Buffer16[1], offset);			
			//Printf ("First Image Width: %ld, Height: %ld, Depth: %ld\n", width, height, depth);

			offset += 12; //offset + 20
			imagesize = (((width+15)&(~15)) * height);
			//imageoffset = (((((width+15)&(~15)) * height) * depth)>>3);
			imageoffset = ((imagesize * depth)>>3);

			//Printf ("First Image Size: %ld, %ld\n", imageoffset, offset);
			offset += imageoffset;
            //Printf("End First Image - Current Offset: %ld\n", offset);
		}
		if(secondimage == 1)
		{
			offset += 4;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
            width = GetUint16(Buffer16); //offset + 4			           
			//Printf ("Second Image Width: %ld, 0x%x%x, %ld\n", width, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			height = GetUint16(Buffer16); //offset + 6			
			//Printf ("Second Image Height: %ld, 0x%x%x, %ld\n", height, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			depth = GetUint16(Buffer16); //offset + 8			
			//Printf ("Second Image Depth: %ld, 0x%x%x, %ld\n", depth, Buffer16[0],Buffer16[1], offset);
            //Printf ("Second Image Width: %ld, Height: %ld, Depth: %ld\n", width, height, depth);
			
			offset += 12; //offset + 20
			imagesize = (((width+15)&(~15)) * height);
			//imageoffset = (((((width+15)&(~15)) * height) * depth)>>3);
			imageoffset = ((imagesize * depth)>>3);						

			//Printf ("Second Image Size: %ld, %ld\n", imageoffset, offset);
			offset += imageoffset;
            //Printf("End Second Image - Current Offset: %ld\n", offset);
		}
		if(deftool == 1)
        {
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
			i = GetUint32(Buffer32);            
			offset += i+4;
			Printf("DefTool - Current Offset: %ld\n", offset);
		}
		if(tooltypes == 1)
		{
			//If tooltypes contains '***' then we have newicon data
            Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
			i = GetUint32(Buffer32)/4-1;
			offset += 4;
            //For a newicon the first tooltype will be 0x29 = " "
            //This will be followed by "***" which is start of newicon data
			
			for(j = 0; j < i; ++j)
            {
				Seek(fh,offset,OFFSET_BEGINNING);
		        Read(fh,Buffer32, 4);
                k = GetUint32(Buffer32);
				offset += k+4;
            }
			//Printf("End ToolTypes - Current Offset: %ld\n", offset);
		}
		if(toolwin == 1)
        {
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
            i = GetUint32(Buffer32);        
		    offset += i+4; 
			Printf("ToolWin - Current Offset: %ld\n", offset);
        }
        /* Try to get new drawer data */
        Seek(fh,offset,OFFSET_BEGINNING);
        Read(fh,Buffer32, 4);
        drawerflags = GetUint32(Buffer32);        
        //offset += 4;

        Seek(fh,offset + 4,OFFSET_BEGINNING);
        Read(fh,Buffer16, 2);
        drawerviewmode = GetUint16(Buffer16);        
        //offset += 2;

		/* Check for Beginning of OS3.5 IFF Data */
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4); //0x464F524D) = 'FORM'		
		if( Buffer32[0] == 0x46 && Buffer32[1] == 0x4F && Buffer32[2] == 0x52 && Buffer32[3] == 0x4D)
		{		
		    //Printf("Found Beginning of OS3.5 IFF Data: %ld\n", offset);			
            Printf("First Offset: %ld\n", offset);
            return offset;
		}
		else
		{
			/* Check Again! */
			offset += 6; //Skip 6 bytes at end for new drawer data
			Seek(fh,offset,OFFSET_BEGINNING);
			Read(fh,Buffer32, 4);
			if( Buffer32[0] == 0x46 && Buffer32[1] == 0x4F && Buffer32[2] == 0x52 && Buffer32[3] == 0x4D)
		    {		
		        //Printf("Found Beginning of OS3.5 IFF Data: %ld\n", offset);
                Printf("Second Offset: %ld\n", offset);
				return offset;
		    }
		}
	}
	Close(fh);	
    Printf("Last Offset: %ld\n", offset);
    if(offset >= imSize) offset = 0;
	return offset;
}

/**************************************************************************************************/


/****************************************************************************/
/*                         SAVE ICON IMAGE FUNCTIONS                        */
/****************************************************************************/


/**************************************************************************************************/

//int ChunkyToBitplanes(UBYTE *chunkyBuffer, UBYTE *bitBuffer, LONG bitPlanes, LONG imagewidth)
int ChunkyBytesToBitplanes(UBYTE *chunkyBuffer, UBYTE *bitBuffer, LONG bitPlanes, LONG imagewidth) 
{    
    int i, j;
    int hOffset = 0; //scanline offsets
    int bitplaneIndex = 0; //Bitplane index
    //int alignedwidth = (imagewidth + 15) & ~15; //Modulus 16   
    int bytesPerLine = (imagewidth / 8); //Bitplane length
    int rowSize = (bytesPerLine * bitPlanes); //Planar scanline length

    /* Converts one scanline from chunky to planar */

    //Printf("ChunkyBytesToBitplanes");
    
    //Block of 8 bytes    
    UBYTE colorBytes[8];   
    
        
    while (hOffset < bytesPerLine)
    {
        //Copy 8 bytes to new buffer. Chunky Buffer is Modulus 16 & therefore Modulus 8.
        memcpy(colorBytes, chunkyBuffer + (hOffset * 8), 8);       
        for (j = 0; j < bitPlanes; j++) //Fill bitplane byte locations (Up to bytesPerLine)
        {
            bitplaneIndex = (j * bytesPerLine);
            //UBYTE bitmask = (UBYTE)(1 << bit);
            
            for (i = 0; i < 8; i++) //Set corresponding bits in bitplane buffers.
            {
                /* Check if the current bit of the index value is set */
                if ((colorBytes[i] & (1 << j)) != 0)
                {                    
                    /* (Bit is Set (1)) Modify the current bit in the bitplane byte to be 1 and leave the others untouched */
                    bitBuffer[bitplaneIndex + hOffset] |= (UBYTE)(1 << 7 - i);
                    //Set bit = number |= 1 << x; //return value |= (UBYTE)(1 << pos);
                }
                else
                {                    
                    /* (Bit is Not Set (0)) Modify the current bit in the bitplane byte to be 0 and leave the others untouched */
                    bitBuffer[bitplaneIndex + hOffset] &= (UBYTE)(~(1 << 7 - i));
                    //Clear bit = number &= ~(1 << x); //return value &= (UBYTE)(~(1 << pos));
                }
            }            
        }
        hOffset++;
	
    } //End while loop

    return 0;        
}

/**************************************************************************************************/

static void MakePlanar(unsigned char *inbuffer, unsigned char *outbuffer, LONG width, LONG height, LONG depth)
{
    int y; //Y-Loop iterator
    LONG kOffset, pOffset; //chunky & planar scanline offsets   
    LONG alignedwidth = (width + 15) & ~15; //Chunky scanline length //Modulus 16
    LONG bytesPerRow = (alignedwidth / 8); //Bitplane length
    LONG linelength = (bytesPerRow * depth); //Planar scanline length

    //Printf("MakePlanar");

    /* Set up line buffer to receive chunky data */
    UBYTE chunkyBuffer[alignedwidth];

    /* Set up line buffer to receive planar data */
    UBYTE bitBuffer[linelength];

    /* Set up Y-Loop to process all scanlines */
    for(y = 0; y < height; y++)
    {
        kOffset = (width * y); /* chunky offset */
        memcpy(chunkyBuffer, inbuffer+kOffset, width); /* copy chunky data */       

        /* Convert Chunky ScanLine to Planar ScanLine */	      
        ChunkyBytesToBitplanes(chunkyBuffer, bitBuffer, depth, alignedwidth);

        pOffset = (linelength * y); /* planar offset */
        memcpy(outbuffer+pOffset, bitBuffer, linelength); /* copy planar data */        
    }
}

/**************************************************************************************************/

static BOOL LoadIconImage(char *name, struct IconImage *im)
{    
    Object *DTImage = NULL;
	struct BitMapHeader *bmh;
	struct pdtBlitPixelArray bpa;
    
	DTImage = NewDTObject(	name,
							(DTA_SourceType),	DTST_FILE,
							(DTA_GroupID),		GID_PICTURE,
							(PDTA_Remap),		FALSE,
							(OBP_Precision),	PRECISION_EXACT,
							TAG_DONE);

	if (DTImage)
	{

		if (GetDTAttrs( DTImage,
					PDTA_BitMapHeader, &bmh,
					TAG_END ) == 1)

		{

			/* Picture struct and buffer mem allocation */			
			if (im->ImageData)
			{
				bpa.MethodID = PDTM_READPIXELARRAY;
				bpa.pbpa_PixelData = (APTR)im->ImageData;
				bpa.pbpa_PixelFormat = PBPAFMT_ARGB;
				bpa.pbpa_PixelArrayMod = bmh->bmh_Width * 4;
				bpa.pbpa_Left = 0;
				bpa.pbpa_Top = 0;
				bpa.pbpa_Width = bmh->bmh_Width;
				bpa.pbpa_Height = bmh->bmh_Height;

				DoMethodA( DTImage, (Msg)&bpa );

				im->Width  = bmh->bmh_Width;
				im->Height = bmh->bmh_Height;
                im->Depth = bmh->bmh_Depth;

				DisposeDTObject( DTImage );

				return TRUE;
			}
		}
		DisposeDTObject( DTImage );
	}
	return FALSE;
}

/**************************************************************************************************/

static char *SaveTarga(char *name, struct IconImage *im, int bpp)
{
    /*******************************/
    /*    BASED ON TGA DATATYPE    */
    /*******************************/

    BPTR filehandle;
    int width, height, depth;	
    char *error = NULL;
    char *status = NULL;
	BOOL tgacomp = FALSE;        
	ULONG numcolors = 0;
	ULONG alignwidth = 0;
	
    if (filehandle = Open (name, MODE_NEWFILE))
    {         
        width = im->Width;
        height = im->Height;
        depth = im->Depth;
    
	    /* disallow images with depth not eq to 8, 16, 24 or 32 */
        //if (depth != 8 && depth != 16 && depth != 24 && depth != 32) 
        if (depth != 24 && depth != 32)    
	    {
		    printf("SaveTarga() --- color depth %d, can save only depths of 24 or 32", depth);		    
		    error = "ERROR_NOT_IMPLEMENTED";		    
	    }
	    printf("SaveTarga() --- Picture size %d x %d (x %d bit)\n", width, height, depth);
        
	    //printf("SaveTarga() --- Setup Pixel Data by BitDepth.\n");

	    /* Set Default Data for 24bit */
	    int cmaptype = 0;
	    int imagetype = 2;
	    int cmapdepth = 0;
	    int compressed = 0;
	    //int compressed = 1;        
        
        /* Set compression value */
	    if(compressed == 1) 
	    {
		    imagetype += 8;
		    tgacomp = TRUE;
	    }

        /* Set Values for new TargaHeader */
        UBYTE TGAHDR[18];
        TGAHDR[0] = 0;
        TGAHDR[1] = cmaptype;
        TGAHDR[2] = imagetype;
        TGAHDR[3] = TGAHDR[4] = 0;    
	    TGAHDR[5] = (UBYTE)(numcolors & 0xFF);
        TGAHDR[6] = (UBYTE)(numcolors >> 8);  
        TGAHDR[7] = cmapdepth; 
        TGAHDR[8] = TGAHDR[9] = 0;    
        TGAHDR[10] = TGAHDR[11] = 0; 
        TGAHDR[12] = (UBYTE)(width & 0xFF);
        TGAHDR[13] = (UBYTE)(width >> 8);    
        TGAHDR[14] = (UBYTE)(height & 0xFF);
        TGAHDR[15] = (UBYTE)(height >> 8); 
        TGAHDR[16] = depth; 
        TGAHDR[17] = 0;     
         
	    /* Write new targa header to File */
        if( FWrite( filehandle, TGAHDR, 18, 1 ) != 1 )
        {
            printf("SaveTarga() --- writing targa header failed !\n");
		    error = "NOT OK";
        }
	    printf("SaveTarga() --- writing targa header successful.\n");    
            
        //printf("SaveTarga() --- begin writing picture data\n");	    
        
	    if((depth == 24) || (depth == 32))
        { 
		    printf("SaveTarga() --- writing to file (24/32bit mode).\n");            			            			
		    status = WriteTruecolor(filehandle, im, bpp, tgacomp);            
		    if(status != "OK")		
		    {			    
			    error = "ERROR_OBJECT_WRONG_TYPE";			    
		    }
        }        
        //printf("SaveTarga() --- Normal Exit\n");
    }
    if (filehandle) Close (filehandle);
	
    return error;       
}

/**************************************************************************************************/

static char *WriteTruecolor(BPTR file, struct IconImage *im, int bpp, BOOL tgacomp)
{
    /*******************************/
    /*    BASED ON TGA DATATYPE    */
    /*******************************/

    /*
    ** Write 24 or 32-bit Scanline
    ** stored in a 24/32bit image file
    */
    
	int y; 
    int idx;	
	int width;
	int height;
	int depth;
	int yoffset;    
    LONG length;
	UBYTE *buf;	
	UBYTE a,r,g,b;
    UBYTE *linebuf;	
    char *error = NULL;
    
    width = im->Width;
    height = im->Height;
    depth = im->Depth;
    length = width*bpp; 
	yoffset = height - 1;    		
	
    /* Allocate memory for scanline data */ 
	linebuf = AllocMem(length, MEMF_ANY);
	if(!linebuf)
	{        
		error = "ERROR_NO_FREE_STORE";
        goto exit;		    
    }

    buf = AllocMem(length, MEMF_ANY);
	if (!buf) 
	{
		error = "ERROR_NO_FREE_STORE";		
        goto exit;
	}

    //printf("\nWriteTrueColor(): Beginning Writing. (24/32bit mode)\n");

    /* Write ScanLine Data from Bottom to Top for TGA */       
	for (y=0; y<height; y++) //Use yoffset to write to the bottom of the file.	
    {		
		/* Get single line of image */            
        if(!memcpy(buf, im->ImageData+(length*yoffset), length)) 
        {
            printf("WriteTrueColor() ---  copy scanline %d failed !\n", y);                
		    error = "READPIXELARRAY FAILED";
            goto exit;
        }       

		/* Process scanline data for 24bit or 32bit */
		if (bpp == 3) 
		{						
			/* Reverse 'RGB' to 'BGR' for TGA file for 24bit */				
            for (idx = 0; idx < (width * bpp); idx+=3)
			{
				r = buf[idx];
				g = buf[idx + 1];
				b = buf[idx + 2];
				linebuf[idx] = b;
				linebuf[idx + 1] = g;
				linebuf[idx + 2] = r;                
			}
		} 
		else if (bpp == 4)
		{			
			/* Reverse 'ARGB' to 'BGRA' for TGA file for 32bit */
            for (idx = 0; idx < (width * bpp); idx+=4)
			{	 
				a = buf[idx];
				r = buf[idx + 1];
				g = buf[idx + 2];
				b = buf[idx + 3];
				linebuf[idx] = b;
				linebuf[idx + 1] = g;
				linebuf[idx + 2] = r;
				linebuf[idx + 3] = a;				
			}
		}
        
        if(tgacomp == TRUE)
		{
			if (bpp == 3)
		    {
			    /* Reverse 'RGB' to 'BGR' for TGA file for 24bit */				
                for (idx = 0; idx < (width * bpp); idx+=3)
			    {
				    r = buf[idx];
				    g = buf[idx + 1];
				    b = buf[idx + 2];
				    buf[idx] = b;
				    buf[idx + 1] = g;
				    buf[idx + 2] = r;                
			    }
			}
			else if (bpp == 4)
		    {			
			    /* Reverse 'ARGB' to 'BGRA' for TGA file for 32bit */
                for (idx = 0; idx < (width * bpp); idx+=4)
			    {	 
				    a = buf[idx];
				    r = buf[idx + 1];
				    g = buf[idx + 2];
				    b = buf[idx + 3];
				    linebuf[idx] = b;
				    linebuf[idx + 1] = g;
				    linebuf[idx + 2] = r;
				    linebuf[idx + 3] = a;				
			    }
		    }
			/* Write rle encoded scanline data to the output file */
			printf("targa.datatype/SaveTarga() --- writing rle data line: %d\n", y);
			//WriteRLE(file, buf, bmh->bmh_Width, bpp); //1,2,3,4
		}
		else
		{
            /* Write unencoded scanline data to the output file */
            if( FWrite( file, linebuf, length, 1 ) != 1 )
            {
                printf("targa.datatype/SaveTarga() --- writing picture scanline %d failed !\n", y);            
			    error = "WRITING PICTURE SCANLINE FAILED";
			    goto exit;
            }
        }
        yoffset--;
    }	

	exit:
	/* Free Resources */	
	if(buf) FreeMem(buf, length);    
	if(linebuf) FreeMem(linebuf, length);

	return error;
}

/**************************************************************************************************/

static BOOL SaveIconImage(char *name, struct IconImage *im, char *type)
{
    /* Save Image to PNG File */ 

	Object *DTImage = NULL;
	struct BitMapHeader *bmhd;
	struct dtWrite dtw;
	struct pdtBlitPixelArray dtb;
	FILE *file = NULL;
	BOOL retval = FALSE;    
    
    
    printf("SaveIconImage - Save Image as PNG: %s\n", name);
		
	DTImage = NewDTObject(	(APTR)NULL,
							DTA_SourceType, DTST_RAM,
							DTA_BaseName, (APTR)type,
							PDTA_DestMode, PMODE_V43,
							TAG_DONE);
	if (!DTImage) return(FALSE);
	else
        printf("Success obtaining new datatype object\n");

	if (GetDTAttrs(DTImage,PDTA_BitMapHeader, &bmhd,TAG_DONE))	
	{
		dtb.MethodID = PDTM_WRITEPIXELARRAY;
		dtb.pbpa_PixelData = im->ImageData;
		dtb.pbpa_PixelFormat = PBPAFMT_ARGB;
		dtb.pbpa_PixelArrayMod = im->Width*4;
		dtb.pbpa_Left = 0;
		dtb.pbpa_Top = 0;
		dtb.pbpa_Width = im->Width;
		dtb.pbpa_Height = im->Height;

		bmhd->bmh_Width = im->Width;
		bmhd->bmh_Height = im->Height;
		bmhd->bmh_Depth = 32;
		bmhd->bmh_PageWidth = 320;
		bmhd->bmh_PageHeight = 240;
		
		printf("SaveIconImage: Picture size %d x %d (x %d depth)\n", bmhd->bmh_Width, bmhd->bmh_Height, bmhd->bmh_Depth);

		DoMethodA(DTImage, (Msg) &dtb);		

		/* Write datatype object to file */
		if (file = Open (name,MODE_NEWFILE))
		{
		        printf("Success opening output file\n");
			dtw.MethodID = DTM_WRITE;
			dtw.dtw_GInfo = NULL;
			dtw.dtw_FileHandle = file;
			dtw.dtw_Mode = DTWM_RAW;
			dtw.dtw_AttrList = NULL;

			if (DoMethodA(DTImage, (Msg) &dtw))			
			{
			    printf("Success saving output file.\n");
                            retval = TRUE;
			}
		}
	}

	if (file) Close (file);
	if (DTImage) DisposeDTObject(DTImage);	
	return retval;
}

/**************************************************************************************************/

static BOOL SaveImage(char *name, struct IconImage *im)
{     
    /* Save Image to ILBM File */

    int i;
    LONG transp;
    LONG numpal = 0;    
    LONG ilbmsize = 0;
	LONG formsize = 0;
    LONG numcolors = 0;
    LONG width, height, depth;

    unsigned char *imdata;
    unsigned char *imgbits;    

    width = im->Width;
    height = im->Height;
    depth = im->Depth; //8;
    transp = im->Transparency;
    numcolors = 1<<depth; //256 //No. colors by depth
	numpal = numcolors*3; //Colormap size in bytes     
	     

    UBYTE rgb[numcolors*3];
    for(i = 0; i < im->NumColors; ++i)
    {
        /* Convert Palette to CMAP Byte Array */
        rgb[i * 3] = (UBYTE)im->Palette[i].Red;
        rgb[(i * 3)+1] = (UBYTE)im->Palette[i].Green;
        rgb[(i * 3)+2] = (UBYTE)im->Palette[i].Blue;
    }   
    
    LONG alignedwidth = (im->Width + 15) & ~15; //Modulus 16
    LONG bytesPerRow = (alignedwidth / 8); 
    LONG linelength = (bytesPerRow * depth); //Planar scanline
    LONG planarsize = (linelength * im->Height);

    //Printf ("depth: %ld\n", depth);
    //Printf ("alignedwidth: %ld\n", alignedwidth);
	//Printf ("bytesPerRow: %ld\n", bytesPerRow);
	//Printf ("linelength: %ld\n", linelength);
	//Printf ("planarsize: %ld\n", planarsize);

    //imgbits = (unsigned char *)AllocVec(planarsize, MEMF_ANY);
    //imgbits = AllocMem( planarsize, MEMF_ANY );
    imgbits = AllocMem( planarsize, MEMF_ANY );    

    MakePlanar(im->ImageData, imgbits, width, height, depth);

    //WriteData("Ram Disk:imgdata1.bin", imgbits, planarsize);     
    
    ilbmsize = ( 40 + (numpal+8) + (planarsize+8) );
    formsize = (ilbmsize - 8);

    //Printf ("ILBM Planar File Size: %ld\n", ( 40 + (numpal+8) + (planarsize+8) ));
	//Printf ("ILBM Planar Form Size: %ld\n", ( 40 + (numpal+8) + (planarsize) ));

    //imdata = (unsigned char *)AllocVec(ilbmsize, MEMF_ANY);
    imdata = AllocMem( ilbmsize, MEMF_ANY );    
    

    /* Build new byte array for bmhd */
    unsigned char bmhd[20];
	bmhd[0] =  (width >> 8); //Width
    bmhd[1] =  (width & 0xFF);
    bmhd[2] =  (height >> 8); //Height
    bmhd[3] =  (height & 0xFF);
	bmhd[4] =  bmhd[5] = 0; //Left Offset
    bmhd[6] =  bmhd[7] = 0; //Top Offset
    bmhd[8] =  depth;
	bmhd[9] =  0; //Masking
    bmhd[10] = 0; //Compression //imgfmt
    bmhd[11] = 0; //Padding
    bmhd[12] = (transp>> 8); 
    bmhd[13] = (transp & 0xFF);
    bmhd[14] = 10; //XAspect;
    bmhd[15] = 11; //YAspect;
    bmhd[16] = (width >> 8); //PageWidth
    bmhd[17] = (width & 0xFF);
    bmhd[18] = (height >> 8); //PageHeight
    bmhd[19] = (height & 0xFF);

    /* Build new byte array with ilbm data */
    PutUint32(imdata, 0x464F524D); //ID_FORM 0x464F524D					
	PutUint32(imdata+4, formsize); 
	PutUint32(imdata+8, 0x494C424D); //ID_ILBM 0x494C424D
	PutUint32(imdata+12, 0x424D4844); //ID_BMHD 0x424D4844
	PutUint32(imdata+16, 20);
	memcpy(imdata+20, bmhd, 20);
    PutUint32(imdata+20+20, 0x434D4150); //ID_CMAP 0x434D4150
	PutUint32(imdata+24+20, numpal);
    memcpy(imdata+28+20, rgb, numpal);
    PutUint32(imdata+48+numpal, 0x424F4459); //ID_BODY 0x424F4459    
    PutUint32(imdata+52+numpal, planarsize);  
    memcpy(imdata+56+numpal, imgbits, planarsize);
    
    /* Write ILBM */
    WriteData(name, imdata, ilbmsize);
    Printf("Successfully Saved ILBM File\n");

    /* Free Resources */
    if (imdata) FreeMem( imdata, ilbmsize );
    if (imgbits) FreeMem( imgbits, planarsize );    
    
    return TRUE;
}

/**************************************************************************************************/

static void SetPalette(struct IconImage *im, const struct IconPalette *pal, long num)
{
  long i, j;

  for(j = 0; j < im->NumColors; ++j)
  {
    i = j%num;
    im->Palette[j].Red = pal[i].Red;
    im->Palette[j].Green = pal[i].Green;
    im->Palette[j].Blue = pal[i].Blue;
  }
}

/**************************************************************************************************/

static char *DecodeRLE(unsigned char *srce, unsigned long bytes, unsigned long len, struct IconImage *im)
{
    /**************************************/
    /*    FUNCTION BASED ON UNPACKBITS    */
    /**************************************/

    //https://en.wikipedia.org/wiki/PackBits

    
    int i, idx;
    //int i, idx, cnt;
    unsigned char val;
    unsigned char *dest;
    long out, copy, loop;
    unsigned long bitbuf = 0;    
    
    idx = 0;
    out = 0;
    copy = 0;
    loop = 0;    

    dest = malloc(len * 2);
    if (!dest) return NULL;
    
    for (i=0; i<len; i++)
    {
        //val = srce++;
        bitbuf = (bitbuf<<8)|*(srce++);
        val = (bitbuf &0xFF);
        Printf ("DecodeRLE - RLE initial value: %c\n", val);
    }
    
    while (idx < len && out <= bytes)
    {
        while (!copy && !loop)
        {
            //val = input[idx];
            val = srce[idx]; //load byte
            Printf ("DecodeRLE - RLE actual value: %c\n", val);            

            if (val <= 127)
            {
                (copy = val + 1);
                idx++; //RLE control byte                
                Printf ("DecodeRLE - RLE copy value: %ld\n", copy);                
            }
            else if (val > 128) 
            {
                loop = (256 - val +1 );
                idx++; //RLE control byte
                Printf ("DecodeRLE - RLE loop value: %ld\n", loop);
            }
            else
            continue;
            //Printf ("DecodeRLE - RLE index value: %ld\n", idx);
        }

        while (copy || loop)
        {
            if (loop)
            {
                val = srce[idx]; //load next byte input[idx];
                //Printf ("DecodeRLE - RLE byte value: %ld\n", val);

                for (i=0; i<loop; i++)
                {
                    dest[out+i] = val;
                    //im->ImageData[out+i] = val;
                }
                idx++; //RLE loop byte                
                out += loop;
                loop = 0;
            }
            if (copy)
            {
                for (i=0; i<copy; i++)
                {
                    val = srce[idx]; //load single byte input[idx];
                    //Printf ("DecodeRLE - RLE byte value: %ld\n", val);                    
                    dest[out+i] = val;
                    //im->ImageData[out+i] = val;
                    idx++; //RLE copy byte
                }
                out += copy;
                copy = 0;
            }
        }
    } //end while idx    
    Printf ("DecodeRLE - RLE out value: %ld\n", out);
    Printf ("DecodeRLE - RLE bytes value: %ld\n", bytes);

    //return out != bytes ? "error decoding iff icon data" : 0;
    return dest;
}

/**************************************************************************************************/

static char *DecodeBytes(unsigned char *srce, unsigned long bytes, unsigned long depth, struct IconImage *im, unsigned long len)
{
    /*************************************/
    /*    FUNCTION BASED ON DECODERLE    */
    /*************************************/       
    
    int mask;
    long bits;
    int i, idx;    
    unsigned char val;
    //unsigned char *dest;
    long out, copy, loop; 
    unsigned long bitbuf = 0;   
    
    idx = 0;
    out = 0;
    copy = 0;
    loop = 0; 
    bits = 0;   

    //dest = malloc(len * 2);
    //if (!dest) return NULL;    

    //Printf ("DecodeBytes - Number of Bitpanes: %ld\n", depth);
    mask = (1<<depth)-1; //numcolors - 1
    
    while (out <= bytes)
    {
        while (!copy && !loop)
        {
            //val = input[idx];
            //val = srce[idx]; //load byte
            if(bits < 8)
            {
                bitbuf = (bitbuf<<8)|*(srce++); //load byte input[idx];
                //Printf ("DecodeBytes - Uppder bitbuffer value: %d\n", (unsigned char)bitbuf);               
                bits += 8;
            }
            //Printf ("DecodeBytes - RLE bits value: %ld\n", bits);
            val = (bitbuf>>(bits-8))&0xFF;  //get byte with mask
            bits -= 8;
            //Printf ("DecodeBytes - RLE actual value: %d\n", val);            

            if (val <= 127)
            {
                (copy = val + 1);
                idx++; //RLE control byte                
                //Printf ("DecodeBytes - RLE copy value: %ld\n", copy);                
            }
            else if (val > 128) 
            {
                loop = (256 - val +1 );
                idx++; //RLE control byte
                //Printf ("DecodeBytes - RLE loop value: %ld\n", loop);
            }
            else
            continue;
            //Printf ("DecodeBytes - RLE index value: %ld\n", idx);
        }

        while (copy || loop)
        {
            if (loop)
            {
                //val = srce[idx]; //load next byte
                if(bits < depth)
                {
                    bitbuf = (bitbuf<<8)|*(srce++); //load byte input[idx];
                    //Printf ("DecodeBytes - Lower bitbuffer value: %d\n", (unsigned char)bitbuf);               
                    bits += 8;
                }                
                val = (bitbuf>>(bits-depth))&mask; //get byte with mask
                //Printf ("DecodeBytes - RLE byte value: %ld\n", val);

                for (i=0; i<loop; i++)
                {                    
                    //dest[out+i] = val;
                    im->ImageData[out+i] = val;
                }
                idx++; //RLE loop byte 
                bits -= depth;               
                out += loop;                
                loop = 0;
            }
            if (copy)
            {
                for (i=0; i<copy; i++)
                {
                    //val = srce[idx]; //load single byte
                    if(bits < depth)
                    {
                        bitbuf = (bitbuf<<8)|*(srce++); //load byte input[idx]
                        //Printf ("DecodeBytes - Lower bitbuffer value: %d\n", (unsigned char)bitbuf);               
                        bits += 8;
                    }                
                    val = (bitbuf>>(bits-depth))&mask; //get byte with mask
                    //Printf ("DecodeBytes - RLE byte value: %ld\n", val);
                    
                    //dest[out+i] = val;
                    im->ImageData[out+i] = val;

                    idx++; //RLE copy byte
                    bits -= depth;
                }
                out += copy;
                copy = 0;
            }
        }
    } //end while idx    
    //Printf ("DecodeBytes - RLE out value: %ld\n", out);
    //Printf ("DecodeBytes - RLE bytes value: %ld\n", bytes);

    //return out != bytes ? "error decoding iff icon data" : 0;
    //return dest;
    return 0;
}

/**************************************************************************************************/

static char *DecodeNew(unsigned char *buffer, long numbytes, long bits, struct IconImage *im, enum DecodeType type, long mode)
{
    /**************************************/
    /*    FUNCTION BASED ON MODIFYICON    */
    /**************************************/
  
  int cop = 0, loop = 0, entries, numbits = 0, mask, curentry = 0;
  unsigned long bitbuf = 0;
  unsigned char byte;
  
  //if(type == DECODE_IMAGE) Printf ("DECODE_IMAGE\n");
  //if(type == DECODE_PALETTE) Printf ("DECODE_PALETTE\n");

  if(!mode) /* no RLE */
    cop = numbytes*8/bits;
  if(type == DECODE_PALETTE)
    entries = im->NumColors*3;
  else
    entries = im->Width*im->Height;

  mask = (1<<bits)-1;

  while((numbytes || (cop && numbits >= bits)) && (curentry < entries))
  {
    if(!cop)
    {
      if(numbits < 8)
      {
        bitbuf = (bitbuf<<8)|*(buffer++); --numbytes;
        numbits += 8;
      }
      byte = (bitbuf>>(numbits-8))&0xFF;
      numbits -= 8;
      if(byte <= 127)     cop = byte+1;
      else if(byte > 128) loop = 256-byte+1;
      else                continue;
    }
    if(cop) ++loop;

    if(numbits < bits)
    {
      bitbuf = (bitbuf<<8)|*(buffer++); --numbytes;
      numbits += 8;
    }
    byte = (bitbuf>>(numbits-bits))&mask;
    while(loop && (curentry < entries))
    {
      if(type == DECODE_PALETTE)
      { /* NOTE: This may not be very fast, but for sure portable */
        switch(curentry%3)
        {
        case 0: im->Palette[curentry/3].Red = byte; break;
        case 1: im->Palette[curentry/3].Green = byte; break;
        case 2: im->Palette[curentry/3].Blue = byte; break;
        }
      }
      else
        im->ImageData[curentry] = byte;
      ++curentry;
      --loop;
    }
    if(cop) --cop;
    numbits -= bits;
  }
  return curentry != entries ? "error decoding new icon data" : 0;
}

/**************************************************************************************************/

static void RemoveToolType(unsigned char **tt)
{
    /**************************************/
    /*    FUNCTION BASED ON MODIFYICON    */
    /**************************************/

    do
    {
        tt[0] = tt[1];
        ++tt;
    } while(*tt);
}

/**************************************************************************************************/

static char *DecodeNewIcon(unsigned char **tt, struct IconImage *im, enum DecodeType type, unsigned char num)
{
    /**************************************/
    /*    FUNCTION BASED ON MODIFYICON    */
    /**************************************/

    long numbits = 0, bits, curentry = 0, entries, bitbuf = 0, loop = 0, mask, val;
    unsigned char byte;
    char *src;    
    
    if(type == DECODE_PALETTE)
    {
        //Printf ("Decode New Icon Image: %d, DECODE_PALETTE\n", num);
        src = *tt+9; /* Tooltype + 9 bytes */
        RemoveToolType(tt);
        entries = im->NumColors*3;
        bits = 8;
    }
    else
    {
        //Printf ("Decode New Icon Image: %d, DECODE_IMAGE\n", num);
        src = ""; /* a dummy start */
        entries = im->Width*im->Height;
        bits = 0;
        for(mask = im->NumColors-1; mask; mask >>= 1)
            ++bits;
        loop = 0;
    }
    mask = (1<<bits)-1;

    while(curentry < entries)
    {
        if(loop) 
        { 
            byte = 0; --loop; 
        }
        else
        {
            if(!*src)
            {
                src = *tt;
                if(!src || src[0] != 'I' || src[1] != 'M' || src[2] != num || src[3] != '=')
                  return "NewIcon data truncated";
                else
                {
                  src += 4; numbits = 0;
                }
                RemoveToolType(tt);
            }
            byte = *(src++);
            if(!byte)            { return "NewIcon data invalid"; }
            else if(byte < 0xA0) { byte -= 0x20; }
            else if(byte < 0xD1) { byte -= 0x51; }
            else                 { loop = byte - 0xD1; byte = 0; }
        }
        bitbuf = (bitbuf<<7)+byte;
        numbits += 7;

        while(numbits >= bits && curentry < entries)
        {
            val = (bitbuf>>(numbits-bits))&mask;

            if(type == DECODE_PALETTE)
            { 
                switch(curentry%3)
                {
                    case 0: im->Palette[curentry/3].Red = val; break;
                    case 1: im->Palette[curentry/3].Green = val; break;
                    case 2: im->Palette[curentry/3].Blue = val; break;
                }
            }
            else
            im->ImageData[curentry] = val;

            numbits -= bits;
            curentry++;
        }
    }
    return 0;
}

/**************************************************************************************************/

static char *Process_NewIconImage(unsigned char **tt_array, struct IconImage *im, unsigned char num)
{
    char *error;    
    unsigned char *b;
    LONG bits = 0, mask, numcolors;

    Printf ("Process New Icon Image\n");

    while(*tt_array)    
    {         
        b = *tt_array;
        if(b[0] == 'I' && b[1] == 'M' && b[2] == num && b[3] == '=')        
        {            
            b += 4;
            im->Transparency = (b[0] == 'B') ? 0 : -1;
            im->Width      = b[1]-0x21;
            im->Height     = b[2]-0x21;
            im->NumColors  = (((b[3]-0x21)<<6)+(b[4]-0x21));
            
            //Estimate depth based on im->NumColors, max depth that fits this many colors
            //4=16,5=32 if colors=16 depth=4, if colors=17 depth=5            
            for(mask = im->NumColors-1; mask; mask >>= 1)
            ++bits;

            im->Depth = bits;            
            mask = (1<<bits)-1;
            numcolors = (1<<bits); //Num colors by depth
            Printf ("Num Colors: %ld\n", numcolors);

            //Printf ("Dimensions: %ld, %ld\n", im->Width, im->Height);
            Printf ("Dimensions: %ld, %ld, %ld\n", im->Width, im->Height, im->Depth);
            Printf ("Transparency: %ld\n", im->Transparency);
            Printf ("Number of Used Colors: %ld\n", im->NumColors);            

            if(im->Width > MAXICONSIZE || im->Height > MAXICONSIZE) //MAXICONSIZE 256 //256x256
                return "icon to large";
            if(im->NumColors > MAXCOLORS)
                return "too many colors";
            
            if((error = DecodeNewIcon(tt_array, im, DECODE_PALETTE, num)))
                return error;
            
            if((error = DecodeNewIcon(tt_array, im, DECODE_IMAGE, num)))
                return error;

            if(im->NumColors > MAXCOLORS) //MAXDEPTH 8 //MAXCOLORS (1<<MAXDEPTH) = 256
                im->NumColors = MAXCOLORS;
        }
        else
        ++tt_array; //Next tooltype
    }
    return 0;
}

/**************************************************************************************************/

//static char *ReadIconImage_NEW(unsigned char *tt_array)
static char *Process_NewIcon(char **tt_array)
{
    char *error;
    //unsigned char **a;
    LONG numimages = 0; 
    struct IconImage *im;    

    Printf ("Process New Icon Data...\n");    
    
    im = (struct IconImage *)AllocMem(sizeof(struct IconImage), MEMF_CLEAR);

    /* Process New Icon Data to Save ILBM Image */      
    if(error = Process_NewIconImage((unsigned char **)tt_array, im, '1')) //This sub function will decode the new icon data
        return error;
    else
    {
        ++numimages; //Number of images
        SaveImage("Ram Disk:img3.iff", im);
    }

    /* Process New Icon Data to Save ILBM Image */       
    if(error = Process_NewIconImage((unsigned char **)tt_array, im, '2')) //This sub function will decode the new icon data
        return error;
    else
    {
        ++numimages; //Number of images
        SaveImage("Ram Disk:img4.iff", im);
    }
    //FreeMem
    if(im) FreeMem(im, sizeof(struct IconImage));
    Printf ("Number of Images Saved: %ld\n", numimages); 
  
    return error;
}

/**************************************************************************************************/

static char *Process_Argb(char *filename, struct IconImage *im, LONG psize, LONG index)
{     
    int i;
    char *ext = NULL;
    char *error = NULL;
    unsigned char *argbimage;    

    /* Setup Disk Object */    
    struct DiskObject *icon = NULL;

    printf("Icon Index: %ld \n", index);

    /* AllocMem */
    argbimage = (unsigned char *)AllocMem(psize, MEMF_CLEAR);    

    /* Get DiskObject (Icon) from srcename */
    ext = strrchr(filename, '.');
	if (ext && !Stricmp(ext, ".info")) 
    	*ext = 0;
    printf("Icon FileName: %s \n", filename);	 
    icon = GetDiskObject(filename);
	if (!icon)
		error = ("icon disk object failed");    
    
    /* Use Icon Library to get ARGB Data */
    if(index == 0)
    {
        IconControl(icon, ICONCTRLA_GetARGBImageData1, &argbimage, TAG_END);
    }
    if(index == 1)
    {
        IconControl(icon, ICONCTRLA_GetARGBImageData2, &argbimage, TAG_END);
    }
    /* For width*height*4 copy ArgbImage to im->ImageData */
    if (argbimage)
    {
        printf ("Successfully decoded ARGB chunk\n");
        for (i=0; i<psize; i++)
        {
            im->ImageData[i] = argbimage[i];
        }
    }
    else
        error = ("Decoding ARGB chunk failed");  

    /* FreeMem */
    //if (argbimage) FreeMem( argbimage, psize );
    FreeDiskObject(icon);    

    return error;
}

/**************************************************************************************************/

LONG Process_PNGIcon(char *name, int *index, LONG pOffset, LONG endFile)
{
    /************************************/
    /*    FUNCTION BASED ON ICON2PNG    */
    /************************************/

    BPTR file;    
    int curPos;
    int offset;         
    LONG pSize;
    LONG wd,ht;      
    UBYTE fileID[4];
    UBYTE chunkID[5];
    UBYTE chunkSize[4];

	if(name) 
	{        
        Printf("\nReading PNG File...\n");
                      
        file=Open(name, MODE_OLDFILE);
        if(file)         
        {  
            offset = pOffset+1;            
	        Seek(file,offset,OFFSET_BEGINNING);
	        Read(file,fileID, 3);
	        fileID[3] = ('\0');
	   
            /* Read File Signature 'PNG' */
            if (strcmp(fileID, "PNG") != 0)
            {
                Printf("\nError: Invalid FileID: 'PNG' Not Found!\n");
                *index = 0;
                return endFile;
            }         

	        /* Find PNG Chunk 'IHDR' */
            offset = pOffset+8;
	        Seek(file,offset,OFFSET_BEGINNING);
	        Read(file,chunkSize, 4);
            chunkSize[4] = ('\0');
            pSize = (chunkSize[0] << 24) | (chunkSize[1] << 16) | (chunkSize[2] << 8) | chunkSize[3];            

            offset = pOffset+12;
	        Seek(file,offset,OFFSET_BEGINNING);
	        Read(file,chunkID, 4);
	        chunkID[4] = ('\0');	        
            
            if (strcmp(chunkID, "IHDR") == 0)
            {           
                printf("Found Critical Chunk: \'%s\'\n", chunkID);
            }
            else
            {
                Printf("\nError: Critical Chunk Not Found!");
                *index = 0;
                return endFile;
            } 
            curPos = (pOffset+16);

            //List all chunks in PNG file & Find IEND            
            while(curPos > 0 && strcmp(chunkID, "IEND") != 0)                        
            {
		        //Find offset to next chunk
		        offset = curPos + pSize + 4;
		        curPos = offset + 8;
	       
		        //Find current chunkSize
		        Seek(file,offset,OFFSET_BEGINNING);
		        Read(file,chunkSize, 4);
		        chunkSize[4] = ('\0');
		        pSize = (chunkSize[0] << 24) | (chunkSize[1] << 16) | (chunkSize[2] << 8) | chunkSize[3];                

		        //Read chunkID from new offset
		        Seek(file,offset + 4,OFFSET_BEGINNING);
		        Read(file,chunkID, 4);
		        chunkID[4] = ('\0');
                printf("Found Chunk: \'%s\'\n", chunkID);

                //Find optional chunk 'ic0n'
		        //if (strcmp(chunkID, "icOn") == 0)
		        //{        
                    //printf("Found Chunk: \'%s\'\n", chunkID);                    
                    //List_Icon_Chunk_Tags(name, file, curPos, pSize);                                        
		        //}

                //Find critical chunk 'IEND'
		        if (strcmp(chunkID, "IEND") == 0)
		        {        
                    printf("Found Critical Chunk: \'%s\'\n", chunkID);
                    *index = 1;
		        }        
	        } //While Not EOF            
            return curPos + 4;                        
        }        
	    Close(file);
    }
    else
    {        
        Printf("PNG Icon File does not exist or cannot be opened.\n");        
    }

    return 0;      
}

/**************************************************************************************************/

BOOL ReadIconImage_PNG(char *srcename)
{
    /************************************/
    /*    FUNCTION BASED ON ICON2PNG    */
    /************************************/
    
    int count = 0;
    int index = 0; 
    LONG imSize = 0;
    LONG filebytes = 0;
    LONG beginning = 0;        
    LONG endOffset = 0;
    LONG nextOffset = 0;
    BOOL retval = FALSE;
    char *outimage = NULL;

    char *image1 = "Ram Disk:Im1.png"; //outPath1
	char *image2 = "Ram Disk:Im2.png"; //outPath2
    char *image3 = "Ram Disk:Im3.png"; //outPath3 

    imSize = GetFileSize(srcename); //Get total filesize 
    Printf( "Size of PNG Icon File: %ld\n", imSize);

    struct IconImage *im;
    im = (struct IconImage *)AllocMem(sizeof(struct IconImage), MEMF_CLEAR);

    while(nextOffset != imSize)
        {            
            nextOffset = Process_PNGIcon(srcename, &index, endOffset, imSize);
            if(nextOffset > 0)
            {                 
                //Printf("\nStart of PNG Image: %ld\n", beginning);                   
                //Printf("\nEnd of PNG Image: %ld\n", nextOffset);
                count++;
                filebytes = (nextOffset - beginning);
                Printf("Size of PNG Image: %ld\n", filebytes);                                         
                outimage = (count > 1) ? image2 : image1;
                if(count > 2) outimage = image3;
                printf("Input File: %s\n", srcename);
                printf("Output File: %s\n", outimage);

                if(filebytes > 0) Printf("Saving PNG Image File...\n");                   
                    
                UBYTE buffer[filebytes];
                BPTR infile = Open(srcename, MODE_OLDFILE);
                Seek(infile,endOffset,OFFSET_BEGINNING);
	            Read(infile,buffer,filebytes);
                BPTR outfile = Open(outimage, MODE_NEWFILE);
                Seek(outfile,0,OFFSET_BEGINNING);
                Write(outfile,buffer,filebytes);

                /* Use datatypes to clear extra chunks */
                LoadIconImage(outfile, im);                
                SaveIconImage(outfile, im, "png");

                if(infile) Close(infile);
                if(outfile) Close(outfile);
                    
                endOffset = nextOffset;
                beginning = nextOffset;                
            }            
        }
        Printf("Number of PNG Images Saved: %d\n", count); //numimages
        if(count > 0) //count = numimages                    
            retval = TRUE;
           
    return retval;
}

/**************************************************************************************************/

BOOL ReadIconImage_IFF(char *srcename, LONG iffoffset)
{    
    ULONG id;
	char *error;	    
    LONG width = 0;
    LONG height = 0;
    LONG psize = 0; //full size
    LONG zsize = 0; //comp size
    LONG cksize = 0; //chunk size
    LONG icsize = 0; //form size
    LONG imSize = 0; //file size
    LONG offset = 0;         
    	        
    BOOL retval = TRUE;
    BOOL ImagPic = FALSE;
    BOOL ArgbPic = FALSE;

	//unsigned char *image;	
	//unsigned char *imdata;
	unsigned char *buffer;
	//unsigned char *imgbuf;
	//unsigned char *palbuf;
    
	LONG numface = 0;    
    LONG numimages = 0, numargb = 0;
    LONG imgfmt, palfmt, numimg, numpal, depth;
	LONG transparency, numcolors, palcolors, icflags;


    /* Get total icon file size */
	imSize = GetFileSize(srcename);
	Printf ("File Size: %ld\n", imSize);

    /* Get byte array of IFF data */
    buffer = (unsigned char *)AllocVec(imSize-iffoffset, MEMF_ANY);
    
    struct IconImage *im;
    struct IconPalette *pal;
    struct IconImageCollection *ico;

    im = (struct IconImage *)AllocMem(sizeof(struct IconImage), MEMF_CLEAR);        
    ico = (struct IconImageCollection *)AllocMem(sizeof(struct IconImageCollection), MEMF_CLEAR);
    pal = (struct IconPalette *)AllocMem(sizeof(struct IconPalette), MEMF_CLEAR);

    /* Open Icon File 0xE310 to Read */    
    BPTR fh = Open(srcename, MODE_OLDFILE);
    if (fh)
    {
        /* First Offset */
        offset += iffoffset;

        /* Beginning of OS3.5 IFF Data */
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh, buffer, imSize);
        Close(fh);

		//0x464F524D = 'FORM' //0x49434F4E = 'ICON'
		//if(GetUint32(buffer) != ID_FORM || GetUint32(buffer+8) != ID_ICON)
		if(GetUint32(buffer) != 0x464F524D || GetUint32(buffer+8) != 0x49434F4E)		
		    retval = FALSE;

		icsize = GetUint32(buffer+4)-4;
		//Printf ("Icon Form Size: %ld\n", icsize);
		buffer += 12;

		LONG ilbmsize = 0;
		LONG formsize = 0;                

		while(icsize > 0)
		{
			id = GetUint32(buffer);
            cksize = GetUint32(buffer+4);
            buffer += 8;
			switch(id)
			{
				case 0x46414345: /* FACE */ //ID_FACE
				    Printf ("Found FACE chunk\n");
					Printf ("Chunk Size: %ld\n", cksize);
				    //if((!numface) != (cksize < 6))
					//FACE chunk must be at least 6 bytes
					if(!numface) //&& cksize >= 6)
					{
						++numface;						
				        width = buffer[0]+1;
                        height = buffer[1]+1;
						Printf ("FACE Dimensions: %ld, %ld\n", width, height);						
					}
				    break;
                case 0x494D4147: /* IMAG */ //ID_IMAG
				    Printf ("\nFound IMAG chunk\n");
					Printf ("Chunk Size: %ld\n", cksize);
					if(cksize >= 10)
					{                       
                        /* IMAG chunk data */                        
                        im = ico->NewImage+numimages;
			            im->Width = width;
                        im->Height = height;
						transparency = buffer[0];						
                        im->NumColors = buffer[1]+1;
                        icflags = buffer[2];
                        imgfmt = buffer[3];
                        palfmt = buffer[4];
                        depth = buffer[5];
                        im->Depth = depth;
                        numimg = GetUint16(buffer+6)+1;
                        numpal = GetUint16(buffer+8)+1; //Size of pal data
						numcolors = 1<<depth; //No. colors by depth						 
						
			            Printf ("Dimensions: %ld, %ld, %ld\n", width, height, depth);
			            //Printf ("Transparency: %ld\n", transparency);
			            Printf ("Num Colors: %ld\n", numcolors);
			            //Printf ("IC Flags: %ld\n", icflags);
			            //Printf ("Image Format: %ld\n", imgfmt);
			            //Printf ("Palette Format: %ld\n", palfmt);
			            //Printf ("Image Depth: %ld\n", depth);
			            //Printf ("Image Size: %ld\n", numimg);
			            //Printf ("Palette Size: %ld\n", numpal);

						LONG datasize = (im->Width*im->Height);
                        LONG alignedwidth = (im->Width + 15) & ~15; //Modulus 16
                        LONG bytesPerRow = (alignedwidth / 8); 
                        LONG linelength = (bytesPerRow * depth); //Planar scanline
                        LONG planarsize = (linelength * im->Height);
												
						if(imgfmt <= 1 && palfmt <= 1) //It's zero or one
						{							
							//Printf ("Decoding Image...\n"); //DecodeOS35
                            //error = DecodeNew(buffer+10, numimg, imgfmt ? depth : 8, im, DECODE_IMAGE, imgfmt);
                            error = DecodeBytes(buffer+10, width*height, imgfmt ? depth : 8, im, numimg);
                            if(error) printf ("Error: %s\n", error);

                            if(icflags & 2)
                            {                                
                                //Printf ("Decoding Palette...\n"); //DecodeOS35
                                error = DecodeNew(buffer+10+numimg, numpal, 8, im, DECODE_PALETTE, palfmt);
                                //error = DecodeBytes(buffer+10+numimg, im->NumColors*3, 8, im, numpal);
                                if(error) printf ("Error: %s\n", error);                                
                            }
                            else if(numimages)
                            {
                                /* Copy 1st Image Palette */
                                im->NumColors = ico->NewImage[0].NumColors;                                
                                SetPalette(im, ico->NewImage[0].Palette, im->NumColors);
                            }                                
						}
					}
					//Error! IMAG chunk must be at least 10 bytes if present.					

					
                    if(numimages == 0)
					{                        
                        ImagPic = TRUE;
                        pal = im->Palette;
                        palcolors = numcolors;
                        //Printf ("Saving ILBM File...\n");                               
                        SaveImage("Ram Disk:img1.iff", im);                                               
					}
					if(numimages == 1)
					{						                        
                        //Printf ("Saving ILBM File...\n");
                        SaveImage("Ram Disk:img2.iff", im);                                               
					}		
					++numimages; //Number of os35 images present in icon file
					break;
                case 0x41524742: /* ARGB */ //ID_ARGB
                    Printf ("Found ARGB chunk\n");
					Printf ("Chunk Size: %ld\n", cksize);
                    if(GetUint32(buffer)==1) //Version
                    {                        
                        /* Size of Compressed Data */
                        zsize = GetUint32(buffer+4);
                        
                        Printf("ARGB compressed size is... %ld.\n", zsize);
                        
                        im->Width = width;
                        im->Height = height;
                        im->Depth = 32;
                        psize = (width * height * 4);                        
                    }

                    if(numargb == 0)
                    {                        
                        error = Process_Argb(srcename, im, psize, numargb);
                        if(error) 
                            Printf("Failed to export ARGB chunk\n");
                        else
                        {                                                        
                            //error = SaveTarga("Ram Disk:img3.tga", im, im->Depth/8);
                            SaveIconImage("Ram Disk:img3.png", im, "png"); //Use datatype to save PNG
                        }                                               
                    }
                    if(numargb == 1)
                    {
                        error = Process_Argb(srcename, im, psize, numargb);
                        if(error) 
                            Printf("Failed to export ARGB chunk\n");
                        else
                        {                                                        
                            //error = SaveTarga("Ram Disk:img4.tga", im, im->Depth/8);
                            SaveIconImage("Ram Disk:img4.png", im, "png"); //Use datatype to save PNG
                        }                                             
                    }
                    ++numargb; //Number of argb images present in icon file
                    break;
				default:
				    break;
			}
			if(cksize & 1) ++cksize;
            buffer += cksize;
            icsize -= 8+cksize;

            /* Style */
            if(ImagPic)
            {
                if(ArgbPic)
                    Printf ("Icon Style: ARGB\n");
                    //ico->IconStyle = "argb";
                else
                    Printf ("Icon Style: OS35\n");
                    //ico->IconStyle = "os35";
                       
            }
            else
            {
                if(ArgbPic)
                    Printf ("Icon Style: OS41\n");
                    //ico->IconStyle = "os41";
            }
		}
		if(icsize < 0)
            retval = FALSE;
    }    
	/* Free Resources */
    FreeMem(ico, sizeof(struct IconImageCollection));
    Printf ("Number of OS35 Images: %ld\n", numimages);
    Printf ("Number of ARGB Images: %ld\n", numargb);       

    return retval;
}

/**************************************************************************************************/

int getarguments(int argc, char **argv)
{
    int retvalue = 0;

    if (argc == 0)
        retvalue = 0;
	if (argc == 1)
        retvalue = 1;
	if (argc == 2)
        retvalue = 2;

    return retvalue;
}

/**************************************************************************************************/

int main(int argc, char **argv)
{	
    char *type = NULL;
    BOOL error = FALSE;
    LONG iffoffset = 0;

	/* CLI Arguments & Switches */
    //iconsplit iconfile.info returns 1
    int action = getarguments(argc, argv);
    //Printf ("Action Value: %d\n", action);    

    if(action == 1 || action == 2)    
    {
        srcename = argv[1]; /* source icon */       
        
        /* Verify 'E310' or 'png' Icon */        
        type = ReadIcon(srcename);
        
        /* Open PeterK's icon.library */
        //if(!(IconBase = OpenLibrary("icon.library",51)))
            //printf("Can't open icon.library v51, v46.4.589!\n");
            //goto exit; //PeterK's Icon Library (46.4.589)

        if(stricmp(type, "iff")==0)
        {
            //printf ("Found Classic Icon File\n");
            printf ("Icon Filename: %s\n", srcename);
            /* Find Beginning of OS3.5 IFF Data */
	        iffoffset = Get_IFFOffset(srcename); 
            if(iffoffset > 0)
            {           
                printf ("\nReturned IFF Offset: %ld\n", iffoffset);
                /* Find IFF IMAG data & Save ILBM */                        		
		        error = ReadIconImage_IFF(srcename, iffoffset);
            }           
            else
            {
                printf ("\nNot a valid OS3.5 Icon File\n");
                printf ("\nSearching for New Icon Data...\n");
                /* Find New Icon Data in Tooltypes */
                LONG ttoffset = Get_Tooltypes(srcename);
                //Printf ("\nFound New Icon Data in Tooltypes: %ld\n", ttoffset);                
            }            
        }
        else if(stricmp(type, "png")==0)
        {
            printf ("Found PNG Icon File\n");
            ReadIconImage_PNG(srcename);
        }
	    else
	    {
            Printf ("Not a valid Icon File\n");
	    }
    }
    else
	{
        Printf ("Not a valid argument\n");
	}
	
	//exit:
    //if(IconBase)  CloseLibrary(IconBase);

	return 0;
}

/**************************************************************************************************/
